package com.kedacom.musicplayer;

import android.Manifest;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.drawable.AnimatedVectorDrawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.IBinder;
import android.support.design.widget.FloatingActionButton;
import android.support.design.widget.NavigationView;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.GravityCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ProgressBar;

import java.util.List;

public class MainActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener,
        ListFragment.ListFragmentInterface,
        PlayerFragment.playerControlInterface
{

    private PlayerService playerService;
    private Intent playIntent;
    private Boolean playerBound = false;
    private ServiceConnection playerConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName componentName, IBinder iBinder) {
            PlayerService.PlayerServiceBinder binder =
                    (PlayerService.PlayerServiceBinder) iBinder;
            playerService = binder.getService();
            playerBound = true;
            Log.d(TAG, "onServiceConnected: " + Boolean.toString(playerBound));
        }

        @Override
        public void onServiceDisconnected(ComponentName componentName) {
            playerBound = false;
        }
    };

    private static final String TAG = "MainActivity";
    private static final int READ_EXTERNAL_REQUEST_ID = 0;
    private static final Integer DEFAULT_NAV_MENU = R.id.nav_player;
    private Integer CURRENT_NAV_MENU = null;
    private NavigationView navigationView = null;
    private DrawerLayout drawer = null;
    private static FloatingActionButton fab = null;
    private static ProgressBar progressBar = null;
    private String currentPlayingFilePath = null;

    private class UpdateProgressBarTask extends AsyncTask<Void, Void, Void> {

        // TODO: update progress bar max at runtime
        @Override
        protected Void doInBackground(Void... voids) {
            while (true) {
                Log.d(TAG, "doInBackground: ...");
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                if (progressBar != null
                        && playerService != null
                        && (playerService.isPlaying() || playerService.isPaused())) {

                    double progress = (double) getCurrentPosition() / getCurrentDuration() * 500;
                    progressBar.setProgress((int) progress);
                    Log.d(TAG, "doInBackground: updating..." + progress);
                } else break;
            }
            return null;
        }

        @Override
        protected void onPostExecute(Void aVoid) {
            super.onPostExecute(aVoid);
            if (progressBar != null)
                progressBar.setProgress(0);
        }
    }

    static final String appPrefFile = "com.kedacom.musicplayer.prefs";
    static SharedPreferences appPreferences;
    static SharedPreferences.Editor appPrefEditor;
    static final String externalStoragePreferenceKey = "READ_EXTERNAL_REQUEST_SHOULD_SHOW";
    static final String currentAbsWorkingPathKey = "CURRENT_ABS_WORKING_PATH";
    static final String BROADCAST_PLAYING_END = "com.kedacom.musicplayer.PlayingEnd";
    static final String BROADCAST_PLAYING_START_NEW = "com.kedacom.musicplayer.PlayingStartNew";


    private BroadcastReceiver playingEndReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            fab.setImageResource(R.drawable.pause_to_play_animation);
            AnimatedVectorDrawable a = (AnimatedVectorDrawable) fab.getDrawable();
            a.start();
            Log.d(TAG, "playingEndReceive: ends");
        }
    };
    private BroadcastReceiver playingStartNewReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            fab.setImageResource(R.drawable.play_to_pause_animation);
            AnimatedVectorDrawable a = (AnimatedVectorDrawable) fab.getDrawable();
            a.start();
            UpdateProgressBarTask u = new UpdateProgressBarTask();
            u.execute(null, null);
            Log.d(TAG, "playingStartNewReceive: starts");
        }
    };


    public void broadcastRegister() {
        registerReceiver(playingEndReceiver, new IntentFilter(BROADCAST_PLAYING_END));
        registerReceiver(playingStartNewReceiver, new IntentFilter(BROADCAST_PLAYING_START_NEW));
    }

    public void broadcastUnregister() {
        unregisterReceiver(playingEndReceiver);
        unregisterReceiver(playingStartNewReceiver);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        appPreferences = getSharedPreferences(appPrefFile, MODE_PRIVATE);
        appPrefEditor = appPreferences.edit();

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        launchOrResurrectService();

        fab = (FloatingActionButton) findViewById(R.id.fab);
        fab.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (playerService.isPlaying())
                    pause();
                else if (playerService.isPaused())
                    resume();
                Log.d(TAG, "onClick: sending...");
            }
        });

        drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        navigationView = (NavigationView) findViewById(R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);

        // set default navigation menu
        switchNavigationItem(CURRENT_NAV_MENU, false);
        if (CURRENT_NAV_MENU == null)
            navigationView.setCheckedItem(DEFAULT_NAV_MENU);
        else
            navigationView.setCheckedItem(CURRENT_NAV_MENU);

        requestPermissions();
        broadcastRegister();
    }


    public void switchNavigationItem(Integer itemId, boolean addToBackStack) {
        Fragment f = null;
        String title = getString(R.string.app_name);
        try {
            if (itemId.equals(CURRENT_NAV_MENU)) {
                drawer.closeDrawer(GravityCompat.START);
                return;
            }
            switch (itemId) {
                case R.id.nav_list:
                    f = new ListFragment();
                    title = getString(R.string.nav_list_string);
                    break;
                case R.id.nav_settings:
                    drawer.closeDrawer(GravityCompat.START);
                    navigationView.setCheckedItem(CURRENT_NAV_MENU);
                    break;
                case R.id.nav_share:
                    drawer.closeDrawer(GravityCompat.START);
                    navigationView.setCheckedItem(CURRENT_NAV_MENU);
                    break;
                case R.id.nav_send:
                    drawer.closeDrawer(GravityCompat.START);
                    navigationView.setCheckedItem(CURRENT_NAV_MENU);
                    break;
                default:
                    f = new PlayerFragment();
                    title = getString(R.string.nav_player_string);
            }
        } catch (Exception e) {
            Log.d(TAG, "switchNavigation(): Maybe null itemId received?");
            f = new PlayerFragment();
            title = getString(R.string.nav_player_string);
            itemId = DEFAULT_NAV_MENU;
        }

        if (f != null) {
            FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
            ft.replace(R.id.content_main_frame, f, "nav_menu");
            if (addToBackStack)
                ft.addToBackStack(null);
            ft.commit();
            CURRENT_NAV_MENU = itemId;
        }

        if (getSupportActionBar() != null)
            getSupportActionBar().setTitle(title);

        drawer.closeDrawer(GravityCompat.START);
        return;
    }

    public void requestPermissions() {
        // Permission request and alert dialog.
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.READ_EXTERNAL_STORAGE)
                != PackageManager.PERMISSION_GRANTED &&
                appPreferences.getBoolean(externalStoragePreferenceKey, true)) {
            AlertDialog.Builder builder = new AlertDialog.Builder(this);
            builder.setMessage(R.string.main_request_permission_alert_string)
                    .setPositiveButton(R.string.main_request_permission_alert_confirm_string, new DialogInterface.OnClickListener() {
                        @Override
                        public void onClick(DialogInterface dialogInterface, int i) {
                            ActivityCompat.requestPermissions(MainActivity.this, new String[]{Manifest.permission.READ_EXTERNAL_STORAGE}, READ_EXTERNAL_REQUEST_ID);
                        }
                    });
            builder.setCancelable(false).create().show();
        }
    }

    public Integer getCurrentFragmentId() {
        Fragment f = getSupportFragmentManager().findFragmentByTag("nav_menu");
        if (f instanceof PlayerFragment) {
            return R.id.nav_player;
        } else if (f instanceof ListFragment) {
            return R.id.nav_list;
        } else {
            return DEFAULT_NAV_MENU;
        }
    }

    private void launchOrResurrectService() {
        if (!playerBound || playIntent == null) {
            playIntent = new Intent(this, PlayerService.class);
            startService(playIntent);
            bindService(playIntent, playerConnection, Context.BIND_AUTO_CREATE);
            Log.d(TAG, "launchOrResurrectService: starting service...");
        } else {
            //TODO: Reconnect to running service after activity is killed.
        }
    }

    @Override
    public void onFileSelected(String filePath, List<String> fileList) {
        Log.d(TAG, "onFileSelected: " + filePath);
        if (!playerService.isPlaying()) {
            fab.setImageResource(R.drawable.play_to_pause_animation);
            AnimatedVectorDrawable a = (AnimatedVectorDrawable) fab.getDrawable();
            a.start();
        }
        currentPlayingFilePath = filePath;
        playerService.play(currentPlayingFilePath);
        playerService.currentPlayList = fileList;
        playerService.currentPlayIndex = fileList.indexOf(filePath);
        UpdateProgressBarTask u = new UpdateProgressBarTask();
        u.execute(null, null);
    }

    public void resume() {
        playerService.resume();
        fab.setImageResource(R.drawable.play_to_pause_animation);
        AnimatedVectorDrawable a = (AnimatedVectorDrawable) fab.getDrawable();
        a.start();
    }

    public void pause() {
        playerService.pause();
        fab.setImageResource(R.drawable.pause_to_play_animation);
        AnimatedVectorDrawable a = (AnimatedVectorDrawable) fab.getDrawable();
        a.start();
    }

    @Override
    public void onStartClick() {
        resume();
    }

    @Override
    public void onPauseClick() {
        pause();
    }

    @Override
    public void onNextClick() {
        playerService.next();
        UpdateProgressBarTask u = new UpdateProgressBarTask();
        u.execute(null, null);
    }

    @Override
    public void onPreviousClick() {
        playerService.previous();
        UpdateProgressBarTask u = new UpdateProgressBarTask();
        u.execute(null, null);
    }

    @Override
    public void getProgressBar(ProgressBar progressBar) {
        this.progressBar = progressBar;
    }

    @Override
    public Integer getCurrentDuration() {
        return playerService.getCurrentDuration();
    }

    @Override
    public Integer getCurrentPosition() {
        return playerService.getCurrentPosition();
    }

    @Override
    public Boolean getServiceState() {
        return !(playerService == null);
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        if (CURRENT_NAV_MENU != null)
            outState.putInt("CURRENT_NAV_MENU", CURRENT_NAV_MENU);
        outState.putBoolean("playerBound", playerBound);
    }

    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        super.onRestoreInstanceState(savedInstanceState);
        CURRENT_NAV_MENU = savedInstanceState.getInt("CURRENT_NAV_MENU");
        playerBound = savedInstanceState.getBoolean("playerBound");
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case READ_EXTERNAL_REQUEST_ID: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    return;
                } else {
                    AlertDialog.Builder builder = new AlertDialog.Builder(MainActivity.this);
                    builder.setMessage(R.string.main_request_denied_alert_string);
                    builder.setPositiveButton(R.string.main_request_denied_alert_confirm_string, null);
                    builder.create().show();
                    if (!ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission.READ_EXTERNAL_STORAGE)) {
                        appPrefEditor.putBoolean(externalStoragePreferenceKey, false).commit();
                    }
                }
                break;
            }
            default:
                break;
        }

    }


    @Override
    public void onBackPressed() {
        DrawerLayout drawer = (DrawerLayout) findViewById(R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
            CURRENT_NAV_MENU = getCurrentFragmentId();
            // TODO: update title at back button
            navigationView.setCheckedItem(CURRENT_NAV_MENU);
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    public boolean onNavigationItemSelected(MenuItem item) {
        switchNavigationItem(item.getItemId(), true);
        return true;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (playerBound) {
            unbindService(playerConnection);
        }
        broadcastUnregister();
        Log.d(TAG, "onDestroy(): ok");
    }
}
